'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createHOCc = exports.createHOC = undefined;

var _createRenderFn = require('./createRenderFn');

var _normalizeProps = require('./normalizeProps');

var _normalizeProps2 = _interopRequireDefault(_normalizeProps);

var _vue = require('vue');

var _vue2 = _interopRequireDefault(_vue);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultStrategy = function defaultStrategy(parent, child) {
  return child;
};

var createHOC = exports.createHOC = function createHOC(Component, options, renderOptions) {
  var hoc = {
    props: (0, _normalizeProps2.default)(typeof Component === 'function' ? Component.options.props : Component.props),
    mixins: [],
    name: (Component.name || 'Anonymous') + 'HOC',
    render: (0, _createRenderFn.createRenderFnc)(renderOptions)
  };
  if (options) {
    Object.keys(options).forEach(function (key) {
      var child = options && options[key];
      var parent = hoc[key];
      var strategy = _vue2.default.config.optionMergeStrategies[key] || defaultStrategy;

      if (key === 'props') {
        hoc[key] = strategy(parent, (0, _normalizeProps2.default)(child));
      } else {
        hoc[key] = strategy(parent, child);
      }
    });
  }

  hoc.mixins && hoc.mixins.push({
    created: function created() {
      this.$createElement = this.$parent.$createElement;
    }
  });

  if (hoc.render && hoc.render.curried) {
    hoc.render = hoc.render(Component);
  }

  return hoc;
};

var createHOCc = exports.createHOCc = function createHOCc(options, renderOptions) {
  var curried = function curried(Component) {
    return createHOC(Component, options, renderOptions);
  };
  curried.curried = true;
  return curried;
};

exports.default = createHOC;
//# sourceMappingURL=createHOC.js.map