'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createRenderFnc = exports.createRenderFn = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _normalizeSlots = require('./normalizeSlots');

var _normalizeSlots2 = _interopRequireDefault(_normalizeSlots);

var _normalizeProps = require('./normalizeProps');

var _normalizeProps2 = _interopRequireDefault(_normalizeProps);

var _assign = require('./assign');

var _assign2 = _interopRequireDefault(_assign);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var isObject = function isObject(test) {
  return Object.prototype.toString.call(test) === '[object Object]';
};

var justBindOptions = ['listeners', 'nativeOn', 'scopedSlots'];

var justBindFn = function justBindFn(key) {
  return justBindOptions.indexOf(key) > -1;
};

var getOptionsKeys = function getOptionsKeys(options) {
  return Object.keys(options).concat(['listeners', 'props', 'attrs']).filter(function (k, i, a) {
    return a.indexOf(k) === i;
  });
};

var createOptionHandlers = function createOptionHandlers(originalOptions, keys) {
  var options = {};

  keys.forEach(function (key) {
    var option = originalOptions[key];

    if (!option) {
      options[key] = function (owner) {
        return owner;
      };
      return;
    }

    if (typeof option === 'function') {
      options[key] = option;
      return;
    }

    if ((typeof option === 'undefined' ? 'undefined' : _typeof(option)) === 'object' && isObject(option)) {
      var optionKeys = Object.keys(option);

      if (!optionKeys.some(function (k) {
        return typeof option[k] === 'function';
      })) {
        options[key] = function (owner) {
          return (0, _assign2.default)({}, owner, option);
        };
        return;
      }

      options[key] = function (owner) {
        var _this = this;

        var result = (0, _assign2.default)({}, owner);
        var justBind = justBindFn(key);

        optionKeys.forEach(function (k) {
          var value = option && option[k];

          if (typeof value === 'function') {
            if (justBind) {
              value = value.bind(_this);
            } else {
              value = value.call(_this, owner);
            }
          }
          result[k] = value;
        });
        return result;
      };
      return;
    }

    options[key] = function () {
      return option;
    };
  });

  return options;
};

var preprocessOptions = function preprocessOptions(originalOptions) {
  var keys = getOptionsKeys(originalOptions);
  var options = createOptionHandlers(originalOptions, keys);

  return function (context, isFunctional) {
    var result = {
      on: {},
      props: {},
      attrs: {}
    };

    keys.forEach(function (key) {
      var owner = isFunctional ? context[key] || context.data[key] : context['$' + key];

      var value = options[key].call(context, owner);

      if (key === 'listeners') {
        key = 'on';
      }

      result[key] = value;
    });

    return result;
  };
};

var getUnusedProps = function getUnusedProps(Component, props) {
  var result = {};
  var target = (0, _normalizeProps2.default)(typeof Component === 'function' ? Component.options.props : Component.props);

  Object.keys(props).forEach(function (prop) {
    if (target[prop] === undefined) {
      result[prop] = props[prop];
    }
  });

  return result;
};

var createRenderFn = exports.createRenderFn = function createRenderFn(Component, options) {
  var getData = preprocessOptions(options || {});

  return function renderHoc(h, context) {
    var data = getData(context || this, !!context);
    var scopedSlots = context && context.data && context.data.scopedSlots || this && this.$scopedSlots;
    var slots = context && context.children || this && this.$slots && (0, _normalizeSlots2.default)(this.$slots, this.$parent) || [];
    var unusedProps = getUnusedProps(Component, data.props);

    data.scopedSlots = data.scopedSlots || scopedSlots;
    data.attrs = (0, _assign2.default)({}, unusedProps, data.attrs);

    return h(Component, data, slots);
  };
};

var createRenderFnc = exports.createRenderFnc = function createRenderFnc(options) {
  var curried = function curried(Component) {
    return createRenderFn(Component, options);
  };
  curried.curried = true;
  return curried;
};
//# sourceMappingURL=createRenderFn.js.map